@@ -93,6 +93,11 @@ enum LOG_STREAM
   LOG_DT_CONS,
   LOG_EVENTS,
   LOG_EVENTS_V,
+  LOG_GBODE,
+  LOG_GBODE_V,
+  LOG_GBODE_NLS,
+  LOG_GBODE_NLS_V,
+  LOG_GBODE_STATES,
   LOG_INIT,
   LOG_INIT_HOMOTOPY,
   LOG_INIT_V,
