@@ -83,7 +83,6 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_IMPRK_LS */                     "impRKLS",
   /* FLAG_INITIAL_STEP_SIZE */            "initialStepSize",
   /* FLAG_INPUT_CSV */                    "csvInput",
-  /* FLAG_INPUT_FILE */                   "exInputFile",
   /* FLAG_INPUT_FILE_STATES */            "stateFile",
   /* FLAG_INPUT_PATH */                   "inputPath",
   /* FLAG_IPOPT_HESSE*/                   "ipopt_hesse",
@@ -115,8 +114,8 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_NLS */                          "nls",
   /* FLAG_NLS_INFO */                     "nlsInfo",
   /* FLAG_NLS_LS */                       "nlsLS",
-  /* FLAG_NLS_MAX_DENSITY */              "nlssMaxDensity",
-  /* FLAG_NLS_MIN_SIZE */                 "nlssMinSize",
+  /* FLAG_NLSS_MAX_DENSITY */             "nlssMaxDensity",
+  /* FLAG_NLSS_MIN_SIZE */                "nlssMinSize",
   /* FLAG_NOEMIT */                       "noemit",
   /* FLAG_NOEQUIDISTANT_GRID */           "noEquidistantTimeGrid",
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/        "noEquidistantOutputFrequency",
@@ -136,6 +135,18 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_PORT */                         "port",
   /* FLAG_R */                            "r",
   /* FLAG_DATA_RECONCILE  */              "reconcile",
+  /* FLAG_DATA_RECONCILE_BOUNDARY */      "reconcileBoundaryConditions",
+  /* FLAG_SR */                           "gbm",
+  /* FLAG_SR_CTRL */                      "gbctrl",
+  /* FLAG_SR_ERR */                       "gberr",
+  /* FLAG_SR_INT */                       "gbint",
+  /* FLAG_SR_NLS */                       "gbnls",
+  /* FLAG_MR */                           "gbfm",
+  /* FLAG_MR_CTRL */                      "gbfctrl",
+  /* FLAG_MR_ERR */                       "gbferr",
+  /* FLAG_MR_INT */                       "gbfint",
+  /* FLAG_MR_NLS */                       "gbfnls",
+  /* FLAG_MR_PAR */                       "gbratio",
   /* FLAG_RT */                           "rt",
   /* FLAG_S */                            "s",
   /* FLAG_SINGLE_PRECISION */             "single",
@@ -145,6 +156,7 @@ const char *FLAG_NAME[FLAG_MAX+1] = {
   /* FLAG_DATA_RECONCILE_Sx */            "sx",
   /* FLAG_UP_HESSIAN */                   "keepHessian",
   /* FLAG_W */                            "w",
+  /* FLAG_PARMODNUMTHREADS */             "parmodNumThreads",
 
   "FLAG_MAX"
 };
@@ -202,7 +214,6 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_IMPRK_LS */                     "selects the linear solver of the integration methods: impeuler, trapezoid and imprungekuta",
   /* FLAG_INITIAL_STEP_SIZE */            "value specifies an initial step size for supported solver",
   /* FLAG_INPUT_CSV */                    "value specifies an csv-file with inputs for the simulation/optimization of the model",
-  /* FLAG_INPUT_FILE */                   "value specifies an external file with inputs for the simulation/optimization of the model",
   /* FLAG_INPUT_FILE_STATES */            "value specifies an file with states start values for the optimization of the model",
   /* FLAG_INPUT_PATH */                   "value specifies a path for reading the input files i.e., model_init.xml and model_info.json",
   /* FLAG_IPOPT_HESSE */                  "value specifies the hessian for Ipopt",
@@ -218,7 +229,7 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_LS */                           "value specifies the linear solver method (default: lapack, totalpivot (fallback))",
   /* FLAG_LS_IPOPT */                     "value specifies the linear solver method for ipopt",
   /* FLAG_LSS */                          "value specifies the linear sparse solver method (default: umfpack)",
-  /* FLAG_LSS_MAX_DENSITY */              "[double (default 0.2)] value specifies the maximum density for using a linear sparse solver",
+  /* FLAG_LSS_MAX_DENSITY */              "[double (default " EXPANDSTRING(DEFAULT_FLAG_LSS_MAX_DENSITY) ")] value specifies the maximum density for using a linear sparse solver",
   /* FLAG_LSS_MIN_SIZE */                 "[int (default " EXPANDSTRING(DEFAULT_FLAG_LSS_MIN_SIZE) ")] value specifies the minimum system size for using a linear sparse solver",
   /* FLAG_LV */                           "[string list] value specifies the logging level",
   /* FLAG_LV_TIME */                      "[double list] specifying time interval to allow loging in",
@@ -234,8 +245,8 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_NLS */                          "value specifies the nonlinear solver",
   /* FLAG_NLS_INFO */                     "outputs detailed information about solving process of non-linear systems into csv files.",
   /* FLAG_NLS_LS */                       "value specifies the linear solver used by the non-linear solver",
-  /* FLAG_NLS_MAX_DENSITY */              "[double (default 0.2)] value specifies the maximum density for using a non-linear sparse solver",
-  /* FLAG_NLS_MIN_SIZE */                 "[int (default " EXPANDSTRING(DEFAULT_FLAG_NLS_MIN_SIZE) ")] value specifies the minimum system size for using a non-linear sparse solver",
+  /* FLAG_NLSS_MAX_DENSITY */             "[double (default " EXPANDSTRING(DEFAULT_FLAG_NLSS_MAX_DENSITY) ")] value specifies the maximum density for using a non-linear sparse solver",
+  /* FLAG_NLSS_MIN_SIZE */                "[int (default " EXPANDSTRING(DEFAULT_FLAG_NLSS_MIN_SIZE) ")] value specifies the minimum system size for using a non-linear sparse solver",
   /* FLAG_NOEMIT */                       "do not emit any results to the result file",
   /* FLAG_NOEQUIDISTANT_GRID */           "stores results not in equidistant time grid as given by stepSize or numberOfIntervals, instead the variable step size of dassl or ida integrator.",
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/        "value controls the output frequency in noEquidistantTimeGrid mode",
@@ -254,7 +265,19 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_OVERRIDE_FILE */                "will override the variables or the simulation settings in the XML setup file with the values from the file",
   /* FLAG_PORT */                         "value specifies the port for simulation status (default disabled)",
   /* FLAG_R */                            "value specifies a new result file than the default Model_res.mat",
-  /* FLAG_DATA_RECONCILE */               "Run the DataReconciliation algorithm for constrained equation",
+  /* FLAG_DATA_RECONCILE */               "Run the Data Reconciliation numerical computation algorithm for constrained equations",
+  /* FLAG_DATA_RECONCILE_BOUNDARY */      "Run the Data Reconciliation numerical computation algorithm for boundary condition equations",
+  /* FLAG_SR */                           "Value specifies the chosen solver of solver gbode (single-rate, slow states integrator)",
+  /* FLAG_SR_CTRL */                      "Step size control of solver gbode (single-rate, slow states integrator)",
+  /* FLAG_SR_ERR */                       "Error estimation done by Richardson extrapolation  (-gberr=1) of solver gbode (single-rate, slow states integrator)",
+  /* FLAG_SR_INT */                       "Interpolation method of solver gbode (single-rate, slow states integrator)",
+  /* FLAG_SR_NLS */                       "Non-linear solver method of solver gbode (single-rate, slow states integrator)",
+  /* FLAG_MR */                           "Value specifies the chosen solver of solver gbode (multi-rate, fast states integrator)",
+  /* FLAG_MR_CTRL */                      "Step size control of solver gbode (multi-rate, fast states integrator)",
+  /* FLAG_MR_ERR */                       "Error estimation done by Richardson extrapolation  (-gberr=1) of solver gbode (multi-rate, fast states integrator)",
+  /* FLAG_MR_INT */                       "Interpolation method of solver gbode (multi-rate, fast states integrator)",
+  /* FLAG_MR_NLS */                       "Non-linear solver method of solver gbode (multi-rate, fast states integrator)",
+  /* FLAG_MR_PAR */                       "Define percentage of states for the fast states selection of solver gbode",
   /* FLAG_RT */                           "value specifies the scaling factor for real-time synchronization (0 disables)",
   /* FLAG_S */                            "value specifies the integration method",
   /* FLAG_SINGLE */                       "output in single precision",
@@ -264,6 +287,7 @@ const char *FLAG_DESC[FLAG_MAX+1] = {
   /* FLAG_DATA_RECONCILE_Sx */            "value specifies a csv-file with inputs as covariance matrix Sx for DataReconciliation",
   /* FLAG_UP_HESSIAN */                   "value specifies the number of steps, which keep hessian matrix constant",
   /* FLAG_W */                            "shows all warnings even if a related log-stream is inactive",
+  /* FLAG_PARMODNUMTHREADS */             "[int default: 0] value specifies the number of threads for simulation using parmodauto. If not specified (or is 0) it will use the systems max number of threads. Note that this option is ignored if the model is not compiled with --parmodauto",
 
   "FLAG_MAX"
 };
@@ -343,7 +367,7 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  If one direction fails, the other direction is always used as fallback option.",
   /* FLAG_HOMOTOPY_ON_FIRST_TRY */
   "  If the model contains the homotopy operator, directly use the homotopy method to solve the initialization problem.\n"
-  "  Without this flag, the solver first tries to solve the initialization problem without homotopy and only uses homotopy as fallback option.",
+  "  This is already the default behaviour, this flag can be used to undo the effect of -noHomotopyOnFirstTry",
   /* FLAG_NO_HOMOTOPY_ON_FIRST_TRY */
   "  Disable the use of the homotopy method to solve the initialization problem.\n"
   "  Without this flag, the solver tries to solve the initialization problem with homotopy if the model contains the homotopy-operator.",
@@ -396,8 +420,6 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Value specifies an initial step size, used by the methods: dassl, ida",
   /* FLAG_INPUT_CSV */
   "  Value specifies an csv-file with inputs for the simulation/optimization of the model",
-  /* FLAG_INPUT_FILE */
-  "  Value specifies an external file with inputs for the simulation/optimization of the model.",
   /* FLAG_INPUT_FILE_STATES */
   "  Value specifies an file with states start values for the optimization of the model.",
   /* FLAG_INPUT_PATH */
@@ -435,10 +457,10 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Value specifies the linear sparse solver method",
   /* FLAG_LSS_MAX_DENSITY */
   "  Value specifies the maximum density for using a linear sparse solver.\n"
-  "  The value is a Double with default value 0.2.",
+  "  The value is a Double with default value " EXPANDSTRING(DEFAULT_FLAG_LSS_MAX_DENSITY) ".",
   /* FLAG_LSS_MIN_SIZE */
   "  Value specifies the minimum system size for using a linear sparse solver.\n"
-  "  The value is an Integer with default value 4001.",
+  "  The value is an Integer with default value " EXPANDSTRING(DEFAULT_FLAG_LSS_MIN_SIZE) ".",
   /* FLAG_LV */
   "  Value (a comma-separated String list) specifies which logging levels to\n"
   "  enable. Multiple options can be enabled at the same time.",
@@ -482,12 +504,12 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Outputs detailed information about solving process of non-linear systems into csv files.",
   /* FLAG_NLS_LS */
   "  Value specifies the linear solver used by the non-linear solver:",
-  /* FLAG_NLS_MAX_DENSITY */
+  /* FLAG_NLSS_MAX_DENSITY */
   "  Value specifies the maximum density for using a non-linear sparse solver.\n"
-  "  The value is a Double with default value 0.2.",
-  /* FLAG_NLS_MIN_SIZE */
+  "  The value is a Double with default value " EXPANDSTRING(DEFAULT_FLAG_NLSS_MAX_DENSITY) ".",
+  /* FLAG_NLSS_MIN_SIZE */
   "  Value specifies the minimum system size for using a non-linear sparse solver.\n"
-  "  The value is an Integer with default value 10001.",
+  "  The value is an Integer with default value " EXPANDSTRING(DEFAULT_FLAG_NLSS_MIN_SIZE) ".",
   /* FLAG_NOEMIT */
   "  Do not emit any results to the result file.",
   /* FLAG_NOEQUIDISTANT_GRID */
@@ -540,7 +562,34 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  The default file-name is based on the model name and output format.\n"
   "  For example: Model_res.mat.",
   /* FLAG_DATA_RECONCILE */
-  "  Run the DataReconciliation algorithm for constrained equation",
+  "  Run the Data Reconciliation numerical computation algorithm for constrained equations",
+  /* FLAG_DATA_RECONCILE_BOUNDARY */
+  "  Run the Data Reconciliation numerical computation algorithm for boundary condition equations",
+  /* FLAG_SR */
+  "  Value specifies the chosen solver of solver gbode (single-rate, slow states integrator).",
+  /* FLAG_SR_CTRL */
+  "  Step size control of solver gbode (single-rate, slow states integrator).",
+  /* FLAG_SR_ERR */
+  "  Error estimation done by Richardson extrapolation (-gberr=1) of solver gbode\n"
+  "  (single-rate, slow states integrator).",
+  /* FLAG_SR_INT */
+  "  Interpolation method of solver gbode (single-rate, slow states integrator).",
+  /* FLAG_SR_NLS */
+  "  Non-linear solver method of solver gbode (single-rate, slow states integrator).",
+  /* FLAG_MR */
+  "  Value specifies the chosen solver of solver gbode (multi-rate, fast states integrator).\n"
+  "  Current Restriction: Fully implicit (Gauss, Radau, Lobatto) RK methods are not supported, yet.",
+  /* FLAG_MR_CTRL */
+  "  Step size control of solver gbode (multi-rate, fast states integrator).",
+  /* FLAG_MR_ERR */
+  "  Error estimation done by Richardson extrapolation  (-gberr=1) of solver gbode\n"
+  "  (multi-rate, fast states integrator).",
+  /* FLAG_MR_INT */
+  "  Interpolation method of solver gbode (multi-rate, fast states integrator).",
+  /* FLAG_MR_NLS */
+  "  Non-linear solver method of solver gbode (multi-rate, fast states integrator).",
+  /* FLAG_MR_PAR */
+  "  Define percentage of states for the fast states selection of solver gbode (values from 0 to 1).",
   /* FLAG_RT */
   "  Value specifies the scaling factor for real-time synchronization (0 disables).\n"
   "  A value > 1 means the simulation takes a longer time to simulate.\n",
@@ -560,10 +609,142 @@ const char *FLAG_DETAILED_DESC[FLAG_MAX+1] = {
   "  Value specifies the number of steps, which keep Hessian matrix constant.",
   /* FLAG_W */
   "  Shows all warnings even if a related log-stream is inactive.",
+  /* FLAG_PARMODNUMTHREADS */
+  "  Value specifies the number of threads for simulation using parmodauto. If not specified (or is 0) it will use the systems max number of threads. Note that this option is ignored if the model is not compiled with --parmodauto",
 
   "FLAG_MAX"
 };
 
+const flag_repeat_policy FLAG_REPEAT_POLICIES[FLAG_MAX] = {
+  FLAG_REPEAT_POLICY_FORBID,
+
+  /* FLAG_ABORT_SLOW */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_ALARM */                        FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_CLOCK */                        FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_CPU */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_CSV_OSTEP */                    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_CVODE_ITER */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_CVODE_LMM */                    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DATA_RECONCILE_Cx */            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DAE_MODE */                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DELTA_X_LINEARIZE */            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DELTA_X_SOLVER */               FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_EMBEDDED_SERVER */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_EMBEDDED_SERVER_PORT */         FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MAT_SYNC */                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_EMIT_PROTECTED */               FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DATA_RECONCILE_Eps */           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_F */                            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HELP */                         FLAG_REPEAT_POLICY_REPLACE,
+  /* FLAG_HOMOTOPY_ADAPT_BEND */          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_BACKTRACE_STRATEGY */  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_H_EPS */               FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_MAX_LAMBDA_STEPS */    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_MAX_NEWTON_STEPS */    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_MAX_TRIES */           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_NEG_START_DIR */       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_ON_FIRST_TRY */        FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NO_HOMOTOPY_ON_FIRST_TRY */     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_DEC_FACTOR */      FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_DEC_FACTOR_PRED */ FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_INC_FACTOR */      FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_INC_THRESHOLD */   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_MAX */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_MIN */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_HOMOTOPY_TAU_START */           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDA_MAXERRORTESTFAIL */         FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDA_MAXNONLINITERS */           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDA_MAXCONVFAILS */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDA_NONLINCONVCOEF */           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDA_LS */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDA_SCALING */                  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IDAS */                         FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IGNORE_HIDERESULT */            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IIF */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IIM */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IIT */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_ILS */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IMPRK_ORDER */                  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IMPRK_LS */                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_INITIAL_STEP_SIZE */            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_INPUT_CSV */                    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_INPUT_FILE_STATES */            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_INPUT_PATH */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IPOPT_HESSE*/                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IPOPT_INIT*/                    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IPOPT_JAC*/                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IPOPT_MAX_ITER */               FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_IPOPT_WARM_START */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_JACOBIAN */                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_JACOBIAN_THREADS */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_L */                            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_L_DATA_RECOVERY */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LOG_FORMAT */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LS */                           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LS_IPOPT */                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LSS */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LSS_MAX_DENSITY */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LSS_MIN_SIZE */                 FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_LV */                           FLAG_REPEAT_POLICY_REPLACE,
+  /* FLAG_LV_TIME */                      FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MAX_BISECTION_ITERATIONS */     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MAX_EVENT_ITERATIONS */         FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MAX_ORDER */                    FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MAX_STEP_SIZE */                FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MEASURETIMEPLOTFORMAT */        FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NEWTON_FTOL */                  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NEWTON_MAX_STEP_FACTOR */       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NEWTON_XTOL */                  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NEWTON_STRATEGY */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NLS */                          FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NLS_INFO */                     FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NLS_LS */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NLSS_MAX_DENSITY */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NLSS_MIN_SIZE */                FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NOEMIT */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NOEQUIDISTANT_GRID */           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NOEQUIDISTANT_OUT_FREQ*/        FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NOEQUIDISTANT_OUT_TIME*/        FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NOEVENTEMIT */                  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NO_RESTART */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NO_ROOTFINDING */               FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NO_SCALING */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_NO_SUPPRESS_ALG */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_OPTDEBUGEJAC */                 FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_OPTIMIZER_NP */                 FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_OPTIMIZER_TGRID */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_OUTPUT */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_OUTPUT_PATH */                  FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_OVERRIDE */                     FLAG_REPEAT_POLICY_COMBINE,
+  /* FLAG_OVERRIDE_FILE */                FLAG_REPEAT_POLICY_COMBINE,
+  /* FLAG_PORT */                         FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_R */                            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DATA_RECONCILE  */              FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DATA_RECONCILE_BOUNDARY */      FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_SR */                           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_SR_CTRL */                      FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_SR_ERR */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_SR_INT */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_SR_NLS */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MR */                           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MR_CTRL */                      FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MR_ERR */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MR_INT */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MR_NLS */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_MR_PAR */                       FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_RT */                           FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_S */                            FLAG_REPEAT_POLICY_REPLACE,
+  /* FLAG_SINGLE_PRECISION */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_SOLVER_STEPS */                 FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_STEADY_STATE */                 FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_STEADY_STATE_TOL */             FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_DATA_RECONCILE_Sx */            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_UP_HESSIAN */                   FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_W */                            FLAG_REPEAT_POLICY_FORBID,
+  /* FLAG_PARMODNUMTHREADS */             FLAG_REPEAT_POLICY_FORBID,
+};
+
+
 const int FLAG_TYPE[FLAG_MAX] = {
   FLAG_TYPE_UNKNOWN,
 
@@ -617,7 +798,6 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_IMPRK_ORDER */                  FLAG_TYPE_OPTION,
   /* FLAG_INITIAL_STEP_SIZE */            FLAG_TYPE_OPTION,
   /* FLAG_INPUT_CSV */                    FLAG_TYPE_OPTION,
-  /* FLAG_INPUT_FILE */                   FLAG_TYPE_OPTION,
   /* FLAG_INPUT_FILE_STATES */            FLAG_TYPE_OPTION,
   /* FLAG_INPUT_PATH */                   FLAG_TYPE_OPTION,
   /* FLAG_IPOPT_HESSE */                  FLAG_TYPE_OPTION,
@@ -649,8 +829,8 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_NLS */                          FLAG_TYPE_OPTION,
   /* FLAG_NLS_INFO */                     FLAG_TYPE_FLAG,
   /* FLAG_NLS_LS */                       FLAG_TYPE_OPTION,
-  /* FLAG_NLS_MAX_DENSITY */              FLAG_TYPE_OPTION,
-  /* FLAG_NLS_MIN_SIZE */                 FLAG_TYPE_OPTION,
+  /* FLAG_NLSS_MAX_DENSITY */             FLAG_TYPE_OPTION,
+  /* FLAG_NLSS_MIN_SIZE */                FLAG_TYPE_OPTION,
   /* FLAG_NOEMIT */                       FLAG_TYPE_FLAG,
   /* FLAG_NOEQUIDISTANT_GRID*/            FLAG_TYPE_FLAG,
   /* FLAG_NOEQUIDISTANT_OUT_FREQ*/        FLAG_TYPE_OPTION,
@@ -670,6 +850,18 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_PORT */                         FLAG_TYPE_OPTION,
   /* FLAG_R */                            FLAG_TYPE_OPTION,
   /* FLAG_DATA_RECONCILE */               FLAG_TYPE_FLAG,
+  /* FLAG_DATA_RECONCILE_BOUNDARY */      FLAG_TYPE_FLAG,
+  /* FLAG_SR */                           FLAG_TYPE_OPTION,
+  /* FLAG_SR_CTRL */                      FLAG_TYPE_OPTION,
+  /* FLAG_SR_ERR */                       FLAG_TYPE_OPTION,
+  /* FLAG_SR_INT */                       FLAG_TYPE_OPTION,
+  /* FLAG_SR_NLS */                       FLAG_TYPE_OPTION,
+  /* FLAG_MR */                           FLAG_TYPE_OPTION,
+  /* FLAG_MR_CTRL */                      FLAG_TYPE_OPTION,
+  /* FLAG_MR_ERR */                       FLAG_TYPE_OPTION,
+  /* FLAG_MR_INT */                       FLAG_TYPE_OPTION,
+  /* FLAG_MR_NLS */                       FLAG_TYPE_OPTION,
+  /* FLAG_MR_PAR */                       FLAG_TYPE_OPTION,
   /* FLAG_RT */                           FLAG_TYPE_OPTION,
   /* FLAG_S */                            FLAG_TYPE_OPTION,
   /* FLAG_SINGLE */                       FLAG_TYPE_FLAG,
@@ -678,7 +870,148 @@ const int FLAG_TYPE[FLAG_MAX] = {
   /* FLAG_STEADY_STATE_TOL */             FLAG_TYPE_OPTION,
   /* FLAG_DATA_RECONCILE_Sx */            FLAG_TYPE_OPTION,
   /* FLAG_UP_HESSIAN */                   FLAG_TYPE_OPTION,
-  /* FLAG_W */                            FLAG_TYPE_FLAG
+  /* FLAG_W */                            FLAG_TYPE_FLAG,
+  /* FLAG_PARMODNUMTHREADS */             FLAG_TYPE_OPTION,
+};
+
+const char *GB_METHOD_NAME[RK_MAX] = {
+  /* GB_UNKNOWN = 0 */   "unknown",
+  /* MS_ADAMS_MOULTON */ "adams",
+  /* RK_EXPL_EULER */    "expl_euler",
+  /* RK_IMPL_EULER */    "impl_euler",
+  /* RK_TRAPEZOID */     "trapezoid",
+  /* RK_SDIRK2 */        "sdirk2",
+  /* RK_SDIRK3 */        "sdirk3",
+  /* RK_ESDIRK2 */       "esdirk2",
+  /* RK_ESDIRK3 */       "esdirk3",
+  /* RK_ESDIRK4 */       "esdirk4",
+  /* RK_RADAU_IA_2 */    "radauIA2",
+  /* RK_RADAU_IA_3 */    "radauIA3",
+  /* RK_RADAU_IA_4 */    "radauIA4",
+  /* RK_RADAU_IIA_2 */   "radauIIA2",
+  /* RK_RADAU_IIA_3 */   "radauIIA3",
+  /* RK_RADAU_IIA_4 */   "radauIIA4",
+  /* RK_LOBA_IIIA_3 */   "lobattoIIIA3",
+  /* RK_LOBA_IIIA_4 */   "lobattoIIIA4",
+  /* RK_LOBA_IIIB_3 */   "lobattoIIIB3",
+  /* RK_LOBA_IIIB_4 */   "lobattoIIIB4",
+  /* RK_LOBA_IIIC_3 */   "lobattoIIIC3",
+  /* RK_LOBA_IIIC_4 */   "lobattoIIIC4",
+  /* RK_GAUSS2 */        "gauss2",
+  /* RK_GAUSS3 */        "gauss3",
+  /* RK_GAUSS4 */        "gauss4",
+  /* RK_GAUSS5 */        "gauss5",
+  /* RK_GAUSS6 */        "gauss6",
+  /* RK_MERSON */        "merson",
+  /* RK_MERSONSSC1 */    "mersonSsc1",
+  /* RK_MERSONSSC2 */    "mersonSsc2",
+  /* RK_HEUN */          "heun",
+  /* RK_FEHLBERG12 */    "fehlberg12",
+  /* RK_FEHLBERG45 */    "fehlberg45",
+  /* RK_FEHLBERG78 */    "fehlberg78",
+  /* RK_FEHLBERGSSC1 */  "fehlbergSsc1",
+  /* RK_FEHLBERGSSC2 */  "fehlbergSsc2",
+  /* RK_RK810 */         "rk810",
+  /* RK_RK1012 */        "rk1012",
+  /* RK_RK1214 */        "rk1214",
+  /* RK_DOPRI45 */       "dopri45",
+  /* RK_DOPRISSC1 */     "dopriSsc1",
+  /* RK_DOPRISSC2 */     "dopriSsc2",
+  /* RK_RKSSC */         "rungekuttaSsc"
+};
+
+const char *GB_METHOD_DESC[RK_MAX] = {
+  /* GB_UNKNOWN = 0 */   "unknown",
+  /* MS_ADAMS_MOULTON */ "Implicit multistep method of type Adams-Moulton (order 2)",
+  /* RK_EXPL_EULER */    "Explizit Runge-Kutta Euler method (order 1)",
+  /* RK_IMPL_EULER */    "Implizit Runge-Kutta Euler method (order 1)",
+  /* RK_TRAPEZOID */     "Implicit Runge-Kutta trapezoid method (order 2)",
+  /* RK_SDIRK2 */        "Singly-diagonal implicit Runge-Kutta (order 2)",
+  /* RK_SDIRK3 */        "Singly-diagonal implicit Runge-Kutta (order 3)",
+  /* RK_ESDIRK2 */       "Explicit singly-diagonal implicit Runge-Kutta (order 2)",
+  /* RK_ESDIRK3 */       "Explicit singly-diagonal implicit Runge-Kutta (order 3)",
+  /* RK_ESDIRK4 */       "Explicit singly-diagonal implicit Runge-Kutta (order 4)",
+  /* RK_RADAU_IA_2 */    "Implicit Runge-Kutta method of Radau family IA (order 3)",
+  /* RK_RADAU_IA_3 */    "Implicit Runge-Kutta method of Radau family IA (order 5)",
+  /* RK_RADAU_IA_4 */    "Implicit Runge-Kutta method of Radau family IA (order 7)",
+  /* RK_RADAU_IIA_2 */   "Implicit Runge-Kutta method of Radau family IIA (order 3)",
+  /* RK_RADAU_IIA_3 */   "Implicit Runge-Kutta method of Radau family IIA (order 5)",
+  /* RK_RADAU_IIA_4 */   "Implicit Runge-Kutta method of Radau family IIA (order 7)",
+  /* RK_LOBA_IIIA_3 */   "Implicit Runge-Kutta method of Lobatto family IIIA (order 4)",
+  /* RK_LOBA_IIIA_4 */   "Implicit Runge-Kutta method of Lobatto family IIIA (order 6)",
+  /* RK_LOBA_IIIB_3 */   "Implicit Runge-Kutta method of Lobatto family IIIB (order 4)",
+  /* RK_LOBA_IIIB_4 */   "Implicit Runge-Kutta method of Lobatto family IIIB (order 6)",
+  /* RK_LOBA_IIIC_3 */   "Implicit Runge-Kutta method of Lobatto family IIIC (order 4)",
+  /* RK_LOBA_IIIC_4 */   "Implicit Runge-Kutta method of Lobatto family IIIC (order 6)",
+  /* RK_GAUSS2 */        "Implicit Runge-Kutta method of Gauss (order 4)",
+  /* RK_GAUSS3 */        "Implicit Runge-Kutta method of Gauss (order 6)",
+  /* RK_GAUSS4 */        "Implicit Runge-Kutta method of Gauss (order 8)",
+  /* RK_GAUSS5 */        "Implicit Runge-Kutta method of Gauss (order 10)",
+  /* RK_GAUSS6 */        "Implicit Runge-Kutta method of Gauss (order 12)",
+  /* RK_MERSON */        "Explicit Runge-Kutta Merson method (order 4)",
+  /* RK_MERSONSSC1 */    "Explicit Runge-Kutta Merson method with large stability region (order 1)",
+  /* RK_MERSONSSC2 */    "Explicit Runge-Kutta Merson method with large stability region (order 2)",
+  /* RK_HEUN */          "Explicit Runge-Kutta Heun method (order 2)",
+  /* RK_FEHLBERG12 */    "Explicit Runge-Kutta Fehlberg method (order 2)",
+  /* RK_FEHLBERG45 */    "Explicit Runge-Kutta Fehlberg method (order 5)",
+  /* RK_FEHLBERG78 */    "Explicit Runge-Kutta Fehlberg method (order 8)",
+  /* RK_FEHLBERGSSC1 */  "Explicit Runge-Kutta Fehlberg method with large stability region (order 1)",
+  /* RK_FEHLBERGSSC2 */  "Explicit Runge-Kutta Fehlberg method with large stability region (order 2)",
+  /* RK_RK810 */         "Explicit 8-10 Runge-Kutta method (order 10)",
+  /* RK_RK1012 */        "Explicit 10-12 Runge-Kutta method (order 12)",
+  /* RK_RK1214 */        "Explicit 12-14 Runge-Kutta method (order 14)",
+  /* RK_DOPRI45 */       "Explicit Runge-Kutta method Dormand-Prince (order 5)",
+  /* RK_DOPRISSC1 */     "Explicit Runge-Kutta method Dormand-Prince with large stability region (order 1)",
+  /* RK_DOPRISSC2 */     "Explicit Runge-Kutta method Dormand-Prince with large stability region (order 2)",
+  /* RK_RKSSC */         "Explicit Runge-Kutta method with large stabiliy region (order 1)"
+};
+
+const char *GB_NLS_METHOD_NAME[GB_NLS_MAX] = {
+  /* GB_NLS_UNKNOWN = 0*/ "unknown",
+  /* GB_NLS_NEWTON */     "newton",
+  /* GB_NLS_KINSOL */     "kinsol"
+};
+
+const char *GB_NLS_METHOD_DESC[GB_NLS_MAX] = {
+  /* GB_NLS_UNKNOWN = 0*/ "unknown",
+  /* GB_NLS_NEWTON */     "Newton method, dense",
+  /* GB_NLS_KINSOL */     "SUNDIALS KINSOL: Inexact Newton, sparse"
+};
+
+const char *GB_CTRL_METHOD_NAME[GB_CTRL_MAX] = {
+  /* GB_CTRL_UNKNOWN */   "unknown",
+  /* GB_CTRL_I */         "i",
+  /* GB_CTRL_PI */        "pi",
+  /* GB_CTRL_CNST */      "const"
+};
+
+const char *GB_CTRL_METHOD_DESC[GB_CTRL_MAX] = {
+  /* GB_CTRL_UNKNOWN */   "unknown",
+  /* GB_CTRL_I */         "I controller for step size",
+  /* GB_CTRL_PI */        "PI controller for step size",
+  /* GB_CTRL_CNST */      "Constant step size"
+};
+
+const char *GB_INTERPOL_METHOD_NAME[GB_INTERPOL_MAX] = {
+  /* GB_INTERPOL_UNKNOWN */           "unknown",
+  /* GB_INTERPOL_LIN */               "linear",
+  /* GB_INTERPOL_HERMITE */           "hermite",
+  /* GB_INTERPOL_HERMITE_a */         "hermite_a",
+  /* GB_INTERPOL_HERMITE_b */         "hermite_b",
+  /* GB_INTERPOL_HERMITE_ERRCTRL */   "hermite_errctrl",
+  /* GB_DENSE_OUTPUT */               "dense_output",
+  /* GB_DENSE_OUTPUT_ERRCTRL */       "dense_output_errctrl"
+};
+
+const char *GB_INTERPOL_METHOD_DESC[GB_INTERPOL_MAX] = {
+  /* GB_INTERPOL_UNKNOWN */         "unknown",
+  /* GB_INTERPOL_LIN */             "Linear interpolation (1st order)",
+  /* GB_INTERPOL_HERMITE */         "Hermite interpolation (3rd order)",
+  /* GB_INTERPOL_HERMITE_a */       "Hermite interpolation (only for left hand side)",
+  /* GB_INTERPOL_HERMITE_b */       "Hermite interpolation (only for right hand side)",
+  /* GB_INTERPOL_HERMITE_ERRCTRL */ "Hermite interpolation with error control",
+  /* GB_DENSE_OUTPUT */             "use dense output formula for interpolation",
+  /* GB_DENSE_OUTPUT_ERRCTRL */     "use dense output fomular with error control"
 };
 
 const char *SOLVER_METHOD_NAME[S_MAX] = {
@@ -689,6 +1022,7 @@ const char *SOLVER_METHOD_NAME[S_MAX] = {
   /* S_IMPEULER */      "impeuler",
   /* S_TRAPEZOID */     "trapezoid",
   /* S_IMPRUNGEKUTTA */ "imprungekutta",
+  /* S_GBODE */         "gbode",
   /* S_IRKSCO */        "irksco",
   /* S_DASSL */         "dassl",
   /* S_IDA */           "ida",
@@ -708,6 +1042,7 @@ const char *SOLVER_METHOD_DESC[S_MAX] = {
   /* S_IMPEULER */      "impeuler - Euler - implicit, fixed step size, order 1",
   /* S_TRAPEZOID */     "trapezoid - trapezoidal rule - implicit, fixed step size, order 2",
   /* S_IMPRUNGEKUTTA */ "imprungekutta - Runge-Kutta methods based on Radau and Lobatto IIA - implicit, fixed step size, order 1-6(selected manually by flag -impRKOrder)",
+  /* S_GBODE */         "gbode - generic bi-rate ODE solver - implicit, explicit, step size control, arbitrary order",
   /* S_IRKSCO */        "irksco - own developed Runge-Kutta solver - implicit, step size control, order 1-2",
   /* S_DASSL */         "dassl - default solver - BDF method - implicit, step size control, order 1-5",
   /* S_IDA */           "ida - SUNDIALS IDA solver - BDF method with sparse linear solver - implicit, step size control, order 1-5",
