@@ -35,6 +35,7 @@
 #include <iostream>
 #include "TextAnnotation.h"
 #include "Modeling/Commands.h"
+#include "Options/OptionsDialog.h"
 
 /*!
  * \class TextAnnotation
@@ -49,7 +50,7 @@
 TextAnnotation::TextAnnotation(QString annotation, GraphicsView *pGraphicsView)
   : ShapeAnnotation(false, pGraphicsView, 0, 0)
 {
-  mpComponent = 0;
+  mpElement = 0;
   mpOriginItem = new OriginItem(this);
   mpOriginItem->setPassive();
   // set the default values
@@ -62,8 +63,25 @@ TextAnnotation::TextAnnotation(QString annotation, GraphicsView *pGraphicsView)
   setShapeFlags(true);
 }
 
+TextAnnotation::TextAnnotation(ModelInstance::Text *pText, bool inherited, GraphicsView *pGraphicsView)
+  : ShapeAnnotation(inherited, pGraphicsView, 0, 0)
+{
+  mpElement = 0;
+  mpOriginItem = new OriginItem(this);
+  mpOriginItem->setPassive();
+  mpText = pText;
+  // set the default values
+  GraphicItem::setDefaults();
+  FilledShape::setDefaults();
+  ShapeAnnotation::setDefaults();
+  // set users default value by reading the settings file.
+  ShapeAnnotation::setUserDefaults();
+  parseShapeAnnotation();
+  setShapeFlags(true);
+}
+
 TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, Element *pParent)
-  : ShapeAnnotation(pShapeAnnotation, pParent), mpComponent(pParent)
+  : ShapeAnnotation(pShapeAnnotation, pParent), mpElement(pParent)
 {
   mpOriginItem = 0;
   updateShape(pShapeAnnotation);
@@ -71,10 +89,25 @@ TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, Element *pPare
   applyTransformation();
 }
 
+TextAnnotation::TextAnnotation(ModelInstance::Text *pText, Element *pParent)
+  : ShapeAnnotation(pParent), mpElement(pParent)
+{
+  mpOriginItem = 0;
+  mpText = pText;
+  // set the default values
+  GraphicItem::setDefaults();
+  FilledShape::setDefaults();
+  ShapeAnnotation::setDefaults();
+  // set users default value by reading the settings file.
+  ShapeAnnotation::setUserDefaults();
+  parseShapeAnnotation();
+  applyTransformation();
+}
+
 TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, GraphicsView *pGraphicsView)
   : ShapeAnnotation(true, pGraphicsView, pShapeAnnotation, 0)
 {
-  mpComponent = 0;
+  mpElement = 0;
   mpOriginItem = new OriginItem(this);
   mpOriginItem->setPassive();
   updateShape(pShapeAnnotation);
@@ -84,14 +117,14 @@ TextAnnotation::TextAnnotation(ShapeAnnotation *pShapeAnnotation, GraphicsView *
 }
 
 TextAnnotation::TextAnnotation(Element *pParent)
-  : ShapeAnnotation(0, pParent), mpComponent(pParent)
+  : ShapeAnnotation(0, pParent), mpElement(pParent)
 {
   mpOriginItem = 0;
   // set the default values
   GraphicItem::setDefaults();
   FilledShape::setDefaults();
   ShapeAnnotation::setDefaults();
-  // give a reasonable size to default component text
+  // give a reasonable size to default element text
   mExtents.replace(0, QPointF(-100, -50));
   mExtents.replace(1, QPointF(100, 50));
   setTextString("%name");
@@ -103,7 +136,7 @@ TextAnnotation::TextAnnotation(Element *pParent)
 TextAnnotation::TextAnnotation(QString annotation, LineAnnotation *pLineAnnotation)
   : ShapeAnnotation(0, pLineAnnotation)
 {
-  mpComponent = 0;
+  mpElement = 0;
   mpOriginItem = 0;
   // set the default values
   GraphicItem::setDefaults();
@@ -124,6 +157,31 @@ TextAnnotation::TextAnnotation(QString annotation, LineAnnotation *pLineAnnotati
   }
 }
 
+TextAnnotation::TextAnnotation(ModelInstance::Text *pText, LineAnnotation *pLineAnnotation)
+  : ShapeAnnotation(pLineAnnotation)
+{
+  mpElement = 0;
+  mpOriginItem = 0;
+  mpText = pText;
+  // set the default values
+  GraphicItem::setDefaults();
+  FilledShape::setDefaults();
+  ShapeAnnotation::setDefaults();
+  parseShapeAnnotation();
+  updateTextString();
+  /* From Modelica Spec 33revision1,
+   * The extent of the Text is interpreted relative to either the first point of the Line, in the case of immediate=false,
+   * or the last point (immediate=true).
+   */
+  if (pLineAnnotation->getPoints().size() > 0) {
+    if (pLineAnnotation->getImmediate()) {
+      setPos(pLineAnnotation->getPoints().last());
+    } else {
+      setPos(pLineAnnotation->getPoints().first());
+    }
+  }
+}
+
 /*!
  * \brief TextAnnotation::TextAnnotation
  * Used by OMSimulator FMU ModelWidget\n
@@ -133,7 +191,7 @@ TextAnnotation::TextAnnotation(QString annotation, LineAnnotation *pLineAnnotati
 TextAnnotation::TextAnnotation(GraphicsView *pGraphicsView)
   : ShapeAnnotation(true, pGraphicsView, 0, 0)
 {
-  mpComponent = 0;
+  mpElement = 0;
   mpOriginItem = 0;
   // set the default values
   GraphicItem::setDefaults();
@@ -167,6 +225,7 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
   mExtents.parse(list.at(8));
   // 10th item of the list contains the textString.
   mTextString.parse(list.at(9));
+  mOriginalTextString = mTextString;
   initUpdateTextString();
 
   // 11th item of the list contains the fontSize.
@@ -202,6 +261,49 @@ void TextAnnotation::parseShapeAnnotation(QString annotation)
   }
 }
 
+void TextAnnotation::parseShapeAnnotation()
+{
+  GraphicItem::parseShapeAnnotation(mpText);
+  FilledShape::parseShapeAnnotation(mpText);
+
+  QList<QPointF> extents;
+  ModelInstance::Extent extent = mpText->getExtent();
+  ModelInstance::Point extent1 = extent.getExtent1();
+  ModelInstance::Point extent2 = extent.getExtent2();
+  extents.append(QPointF(extent1.x(), extent1.y()));
+  extents.append(QPointF(extent2.x(), extent2.y()));
+  mExtents = extents;
+  mTextString = mpText->getTextString();
+  mOriginalTextString = mTextString;
+  initUpdateTextString();
+
+  mFontSize = mpText->getFontSize();
+  if (mpText->getTextColor().getColor().isValid()) {
+    mLineColor = mpText->getTextColor().getColor();
+  }
+  if (!mpText->getFontName().isEmpty()) {
+    mFontName = mpText->getFontName().isEmpty();
+  }
+  QStringList textStyles = mpText->getTextStyle();
+  foreach (QString textStyle, textStyles) {
+    if (textStyle == "TextStyle.Bold") {
+      mTextStyles.append(StringHandler::TextStyleBold);
+    } else if (textStyle == "TextStyle.Italic") {
+      mTextStyles.append(StringHandler::TextStyleItalic);
+    } else if (textStyle == "TextStyle.UnderLine") {
+      mTextStyles.append(StringHandler::TextStyleUnderLine);
+    }
+  }
+  QString horizontalAlignment = StringHandler::removeFirstLastQuotes(stripDynamicSelect(mpText->getHorizontalAlignment()));
+  if (horizontalAlignment == "TextAlignment.Left") {
+    mHorizontalAlignment = StringHandler::TextAlignmentLeft;
+  } else if (horizontalAlignment == "TextAlignment.Center") {
+    mHorizontalAlignment = StringHandler::TextAlignmentCenter;
+  } else if (horizontalAlignment == "TextAlignment.Right") {
+    mHorizontalAlignment = StringHandler::TextAlignmentRight;
+  }
+}
+
 /*!
  * \brief TextAnnotation::boundingRect
  * Defines the bounding rectangle of the shape.
@@ -237,18 +339,19 @@ void TextAnnotation::paint(QPainter *painter, const QStyleOptionGraphicsItem *op
   Q_UNUSED(widget);
   //! @note We don't show text annotation that contains % for Library Icons or if it is too long.
   if (mpGraphicsView && mpGraphicsView->isRenderingLibraryPixmap()) {
-    if (mTextString.contains("%") || mTextString.length() > maxTextLengthToShowOnLibraryIcon) {
+    if (mOriginalTextString.contains("%") || mOriginalTextString.length() > OptionsDialog::instance()->getGeneralSettingsPage()->getLibraryIconTextLengthSpinBox()->value()) {
       return;
     }
-  } else if (mpComponent && mpComponent->getGraphicsView()->isRenderingLibraryPixmap()) {
+  } else if (mpElement && mpElement->getGraphicsView()->isRenderingLibraryPixmap()) {
     return;
   }
   if (mVisible) {
     // state machine visualization
-    // text annotation on a component
-    if (mpComponent && mpComponent->getLibraryTreeItem() && mpComponent->getLibraryTreeItem()->isState()
-        && mpComponent->getGraphicsView()->isVisualizationView()) {
-      if (mpComponent->isActiveState()) {
+    // text annotation on a element
+    if (mpElement && mpElement->getGraphicsView()->isVisualizationView()
+        && ((mpElement->getGraphicsView()->getModelWidget()->isNewApi() && mpElement->getModel() && mpElement->getModel()->isState())
+            || (mpElement->getLibraryTreeItem() && mpElement->getLibraryTreeItem()->isState()))) {
+      if (mpElement->isActiveState()) {
         painter->setOpacity(1.0);
       } else {
         painter->setOpacity(0.2);
@@ -360,7 +463,7 @@ void TextAnnotation::drawTextAnnotation(QPainter *painter)
     }
   }
   // draw the font
-  if (mpComponent || mappedBoundingRect.width() != 0 || mappedBoundingRect.height() != 0) {
+  if (mpElement || mappedBoundingRect.width() != 0 || mappedBoundingRect.height() != 0) {
     painter->drawText(mappedBoundingRect, StringHandler::getTextAlignment(mHorizontalAlignment) | Qt::AlignVCenter | Qt::TextDontClip, textToDraw);
   }
 }
@@ -377,7 +480,7 @@ QString TextAnnotation::getOMCShapeAnnotation()
   // get the extents
   annotationString.append(mExtents.toQString());
   // get the text string
-  annotationString.append(mTextString.toQString());
+  annotationString.append(mOriginalTextString.toQString());
   // get the font size
   annotationString.append(mFontSize.toQString());
   // get the text color
@@ -426,7 +529,7 @@ QString TextAnnotation::getShapeAnnotation()
     annotationString.append(QString("extent=%1").arg(mExtents.toQString()));
   }
   // get the text string
-  annotationString.append(QString("textString=%1").arg(mTextString.toQString()));
+  annotationString.append(QString("textString=%1").arg(mOriginalTextString.toQString()));
   // get the font size
   if (mFontSize.isDynamicSelectExpression() || mFontSize != 0) {
     annotationString.append(QString("fontSize=%1").arg(mFontSize.toQString()));
@@ -469,10 +572,10 @@ void TextAnnotation::updateShape(ShapeAnnotation *pShapeAnnotation)
 
 void TextAnnotation::initUpdateTextString()
 {
-  if (mpComponent) {
-    if (mTextString.contains("%")) {
+  if (mpElement) {
+    if (mOriginalTextString.contains("%")) {
       updateTextString();
-      connect(mpComponent, SIGNAL(displayTextChanged()), SLOT(updateTextString()), Qt::UniqueConnection);
+      connect(mpElement, SIGNAL(displayTextChanged()), SLOT(updateTextString()), Qt::UniqueConnection);
     }
   }
 }
@@ -486,52 +589,63 @@ void TextAnnotation::updateTextStringHelper(QRegExp regExp)
 {
   int pos = 0;
   while ((pos = regExp.indexIn(mTextString, pos)) != -1) {
-    QString variable = regExp.cap(0).trimmed(); QString qs;
+    QString variable = regExp.cap(0).trimmed();
     if ((!variable.isEmpty()) && (variable.compare("%%") != 0) && (variable.compare("%name") != 0) && (variable.compare("%class") != 0)) {
       variable.remove("%");
       variable = StringHandler::removeFirstLastCurlBrackets(variable);
       if (!variable.isEmpty()) {
         QString textValue;
         /* Ticket:4204
-         * If we have extend component then call Element::getParameterDisplayString from root component.
+         * If we have extend element then call Element::getParameterDisplayString from root element.
          */
-        textValue = mpComponent->getRootParentComponent()->getParameterDisplayString(variable);
+        textValue = mpElement->getRootParentElement()->getParameterDisplayString(variable);
         if (!textValue.isEmpty()) {
-          QString unit = mpComponent->getRootParentComponent()->getParameterModifierValue(variable, "unit");
-          QString displayUnit = mpComponent->getRootParentComponent()->getParameterModifierValue(variable, "displayUnit");
-          Element *pElement = mpComponent->getRootParentComponent()->getElementByName(variable);
-          if (pElement) {
-            if (displayUnit.isEmpty()) {
-              displayUnit = pElement->getDerivedClassModifierValue("displayUnit");
+          QString unit = mpElement->getRootParentElement()->getParameterModifierValue(variable, "unit");
+          QString displayUnit = mpElement->getRootParentElement()->getParameterModifierValue(variable, "displayUnit");
+          if (MainWindow::instance()->isNewApi()) {
+            ModelInstance::Element* pModelElement = Element::getModelElementByName(mpElement->getRootParentElement()->getModel(), variable);
+            if (pModelElement) {
+              if (displayUnit.isEmpty()) {
+                displayUnit = pModelElement->getModifierValueFromType(QStringList() << "displayUnit");
+              }
+              if (unit.isEmpty()) {
+                unit = pModelElement->getModifierValueFromType(QStringList() << "unit");
+              }
             }
-            if (unit.isEmpty()) {
-              unit = pElement->getDerivedClassModifierValue("unit");
-            }
-            // if display unit is still empty then use unit
-            if (displayUnit.isEmpty()) {
-              displayUnit = unit;
+          } else {
+            Element *pElement = mpElement->getRootParentElement()->getElementByName(variable);
+            if (pElement) {
+              if (displayUnit.isEmpty()) {
+                displayUnit = pElement->getDerivedClassModifierValue("displayUnit");
+              }
+              if (unit.isEmpty()) {
+                unit = pElement->getDerivedClassModifierValue("unit");
+              }
             }
           }
-          // do not do any conversion if unit or displayUnit is empty of if both are 1!
-          if (displayUnit.isEmpty() || unit.isEmpty() || (displayUnit.compare("1") == 0 && unit.compare("1") == 0)) {
-            qs = mTextString.replace(pos, regExp.matchedLength(), textValue);
-            pos += textValue.length();
+          // if display unit is still empty then use unit
+          if (displayUnit.isEmpty()) {
+            displayUnit = unit;
+          }
+          QString textValueWithDisplayUnit;
+          // Do not show displayUnit if value is not a literal constant or if displayUnit is empty or if unit and displayUnit are 1!
+          if (!Utilities::isValueLiteralConstant(textValue) || displayUnit.isEmpty() || (displayUnit.compare("1") == 0 && unit.compare("1") == 0)) {
+            textValueWithDisplayUnit = textValue;
+          } else if (unit.compare(displayUnit) == 0) {  // Do not do any conversion if unit and displayUnit are same.
+            textValueWithDisplayUnit = QString("%1 %2").arg(textValue, Utilities::convertUnitToSymbol(displayUnit));
           } else {
-            QString textValueWithDisplayUnit;
             OMCProxy *pOMCProxy = MainWindow::instance()->getOMCProxy();
             OMCInterface::convertUnits_res convertUnit = pOMCProxy->convertUnits(unit, displayUnit);
             if (convertUnit.unitsCompatible) {
               qreal convertedValue = Utilities::convertUnit(textValue.toDouble(), convertUnit.offset, convertUnit.scaleFactor);
               textValue = StringHandler::number(convertedValue, textValue);
-              displayUnit = Utilities::convertUnitToSymbol(displayUnit);
-              textValueWithDisplayUnit = QString("%1 %2").arg(textValue, displayUnit);
+              textValueWithDisplayUnit = QString("%1 %2").arg(textValue, Utilities::convertUnitToSymbol(displayUnit));
             } else {
-              unit = Utilities::convertUnitToSymbol(unit);
-              textValueWithDisplayUnit = QString("%1 %2").arg(textValue, unit);
+              textValueWithDisplayUnit = QString("%1 %2").arg(textValue, Utilities::convertUnitToSymbol(unit));
             }
-            qs = mTextString.replace(pos, regExp.matchedLength(), textValueWithDisplayUnit);
-            pos += textValueWithDisplayUnit.length();
           }
+          mTextString.replace(pos, regExp.matchedLength(), textValueWithDisplayUnit);
+          pos += textValueWithDisplayUnit.length();
         } else { /* if the value of %\\W* is empty then remove the % sign. */
           mTextString.replace(pos, 1, "");
         }
@@ -557,9 +671,10 @@ void TextAnnotation::updateTextString()
    * of an enumeration type, replace %par by the item name, not by the full name.
    * [Example: if par="Modelica.Blocks.Types.Enumeration.Periodic", then %par should be displayed as "Periodic"]
    * - %% replaced by %
-   * - %name replaced by the name of the component (i.e. the identifier for it in in the enclosing class).
+   * - %name replaced by the name of the element (i.e. the identifier for it in in the enclosing class).
    * - %class replaced by the name of the class.
    */
+  mTextString = mOriginalTextString;
   LineAnnotation *pLineAnnotation = dynamic_cast<LineAnnotation*>(parentItem());
   if (pLineAnnotation) {
     if (mTextString.toLower().contains("%condition")) {
@@ -570,17 +685,15 @@ void TextAnnotation::updateTextString()
         mTextString.prepend(QString("%1: ").arg(pLineAnnotation->getPriority()));
       }
     }
-  } else if (mpComponent) {
-    mTextString.reset();
-
+  } else if (mpElement) {
     if (!mTextString.contains("%")) {
       return;
     }
     if (mTextString.toLower().contains("%name")) {
-      mTextString.replace(QRegExp("%name"), mpComponent->getName());
+      mTextString.replace(QRegExp("%name"), mpElement->getName());
     }
-    if (mTextString.toLower().contains("%class") && mpComponent->getLibraryTreeItem()) {
-      mTextString.replace(QRegExp("%class"), mpComponent->getLibraryTreeItem()->getNameStructure());
+    if (mTextString.toLower().contains("%class")) {
+      mTextString.replace(QRegExp("%class"), mpElement->getClassName());
     }
     if (!mTextString.contains("%")) {
       return;
