@@ -131,7 +131,7 @@ void MetaModelicaEditor::contentsHasChanged(int position, int charsRemoved, int
     }
     /* if user is changing the text. */
     if (!mForceSetPlainText) {
-      mpModelWidget->updateModelText();
+      contentsChanged();
     }
   }
 }
@@ -168,7 +168,11 @@ void MetaModelicaHighlighter::initializeSettings()
   font.setFamily(mpMetaModelicaEditorPage->getOptionsDialog()->getTextEditorPage()->getFontFamilyComboBox()->currentFont().family());
   font.setPointSizeF(mpMetaModelicaEditorPage->getOptionsDialog()->getTextEditorPage()->getFontSizeSpinBox()->value());
   mpPlainTextEdit->document()->setDefaultFont(font);
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 11, 0))
+  mpPlainTextEdit->setTabStopDistance((qreal)(mpMetaModelicaEditorPage->getOptionsDialog()->getTextEditorPage()->getTabSizeSpinBox()->value() * QFontMetrics(font).horizontalAdvance(QLatin1Char(' '))));
+#else // QT_VERSION_CHECK
   mpPlainTextEdit->setTabStopWidth(mpMetaModelicaEditorPage->getOptionsDialog()->getTextEditorPage()->getTabSizeSpinBox()->value() * QFontMetrics(font).width(QLatin1Char(' ')));
+#endif // QT_VERSION_CHECK
   // set color highlighting
   mHighlightingRules.clear();
   HighlightingRule rule;
