@@ -40,6 +40,7 @@
 #include "Logging.h"
 #include "MatReader.h"
 #include "Model.h"
+#include "OMSString.h"
 #include "ResultReader.h"
 #include "Scope.h"
 #include "SignalDerivative.h"
@@ -53,15 +54,15 @@
 #endif
 #include "Types.h"
 #include "Version.h"
+
+#include <chrono>
+#include <condition_variable>
+#include <iostream>
 #include <miniunz.h>
+#include <mutex>
 #include <pugixml.hpp>
 #include <string>
-
-#include <iostream>
-#include <chrono>
 #include <thread>
-#include <mutex>
-#include <condition_variable>
 
 #if defined(OMS_STATIC)
 extern "C"
@@ -177,6 +178,50 @@ oms_status_enu_t oms_delete(const char* cref)
   return model->delete_(tail);
 }
 
+oms_status_enu_t oms_deleteResources(const char* cref_)
+{
+  oms::ComRef tail(cref_);
+  oms::ComRef front = tail.pop_front();
+
+  oms::ComRef modelCref(front);
+  modelCref.pop_suffix();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(modelCref);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  if (tail.isEmpty())
+  {
+    return model->deleteResourcesInSSP(front.pop_suffix());
+  }
+  else
+    return model->deleteReferencesInSSD(tail);
+}
+
+oms_status_enu_t oms_referenceResources(const char* cref_, const char* ssmFile)
+{
+  oms::ComRef tail(cref_);
+  oms::ComRef front = tail.pop_front();
+
+  oms::ComRef modelCref(front);
+  modelCref.pop_suffix();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(modelCref);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->referenceResources(tail, std::string(ssmFile));
+}
+
+oms_status_enu_t oms_reduceSSV(const char* cref, const char* ssvfile, const char* ssmfile, const char* filepath)
+{
+  oms::Model* model = oms::Scope::GetInstance().getModel(cref);
+  if (!model)
+    return logError_ModelNotInScope(cref);
+
+  return model->reduceSSV(ssvfile, ssmfile, filepath);
+}
+
 oms_status_enu_t oms_export(const char* cref, const char* filename)
 {
   return oms::Scope::GetInstance().exportModel(oms::ComRef(cref), std::string(filename));
@@ -215,6 +260,17 @@ oms_status_enu_t oms_list(const char* cref_, char** contents)
   return model->list(tail, contents);
 }
 
+oms_status_enu_t oms_listVariants(const char* cref, char** contents)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->listVariants(tail, contents);
+}
+
 oms_status_enu_t oms_exportSnapshot(const char* cref_, char** contents)
 {
   oms::ComRef tail(cref_);
@@ -279,7 +335,7 @@ oms_status_enu_t oms_importSnapshot(const char* cref, const char* snapshot, char
   return oms::Scope::GetInstance().importSnapshot(oms::ComRef(cref), snapshot, newCref);
 }
 
-oms_status_enu_t oms_addResources(const char* cref_)
+oms_status_enu_t oms_newResources(const char* cref_)
 {
   oms::ComRef tail(cref_);
   oms::ComRef front = tail.pop_front();
@@ -291,7 +347,22 @@ oms_status_enu_t oms_addResources(const char* cref_)
   if (!model)
     return logError_ModelNotInScope(front);
 
-  return model->addResources(tail);
+  return model->newResources(tail);
+}
+
+oms_status_enu_t oms_addResources(const char* cref_, const char* path)
+{
+  oms::ComRef tail(cref_);
+  oms::ComRef front = tail.pop_front();
+
+  oms::ComRef modelCref(front);
+  modelCref.pop_suffix();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(modelCref);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->addResources(front, path);
 }
 
 oms_status_enu_t oms_addSystem(const char* cref_, oms_system_enu_t type)
@@ -441,7 +512,7 @@ oms_status_enu_t oms_getSystemType(const char* cref, oms_system_enu_t* type)
   return oms_status_ok;
 }
 
-oms_status_enu_t oms_addConnection(const char *crefA, const char *crefB)
+oms_status_enu_t oms_addConnection(const char *crefA, const char *crefB, bool suppressUnitConversion)
 {
   oms::ComRef tailA(crefA);
   oms::ComRef modelCref = tailA.pop_front();
@@ -461,7 +532,7 @@ oms_status_enu_t oms_addConnection(const char *crefA, const char *crefB)
     return logError_SystemNotInModel(modelCref, systemCref);
   }
 
-  return system->addConnection(tailA,tailB);
+  return system->addConnection(tailA, tailB, suppressUnitConversion);
 }
 
 oms_status_enu_t oms_deleteConnection(const char *crefA, const char *crefB)
@@ -856,6 +927,46 @@ oms_status_enu_t oms_addSubModel(const char* cref, const char* fmuPath)
   return system->addSubModel(tail, fmuPath);
 }
 
+oms_status_enu_t oms_activateVariant(const char* crefA, const char* crefB)
+{
+  oms::ComRef tail(crefA);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->activateVariant(tail, crefB);
+}
+
+oms_status_enu_t oms_duplicateVariant(const char* crefA, const char* crefB)
+{
+  oms::ComRef tail(crefA);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  return model->duplicateVariant(tail, crefB);
+}
+
+oms_status_enu_t oms_replaceSubModel(const char* cref, const char* fmuPath, bool dryRun, int* warningCount)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  return system->replaceSubModel(tail, fmuPath, dryRun, *warningCount);
+}
+
 oms_status_enu_t oms_getComponentType(const char* cref, oms_component_enu_t* type)
 {
   oms::ComRef tail(cref);
@@ -1058,24 +1169,20 @@ static int do_simulation(std::string model, std::chrono::duration<double> timeou
   return 0;
 }
 
-oms_status_enu_t oms_RunFile(const char* filename_)
+oms_status_enu_t oms_RunFile(const char* filename)
 {
   oms_status_enu_t status;
-  std::string filename(filename_);
-  std::string type = "";
-  if (filename.length() > 4)
-    type = filename.substr(filename.length() - 4);
-  else
-    return logError("Not able to process file '" + filename + "'\nUse OMSimulator --help for more information.");
+  filesystem::path path(filename);
+  std::string type = path.extension().string();
 
   if (type == ".fmu")
   {
     std::string modelName("model");
     std::string systemName = modelName + ".root";
-    std::string fmuName = systemName + ".fmu";
+    std::string fmuName = systemName + (oms::ComRef::isValidIdent(path.stem().string()) ? ("." + path.stem().string()) : ".fmu");
     oms_fmi_kind_enu_t kind;
 
-    status = oms_extractFMIKind(filename.c_str(), &kind);
+    status = oms_extractFMIKind(filename, &kind);
     if (oms_status_ok != status) return logError("oms_extractFMIKind failed");
 
     status = oms_newModel(modelName.c_str());
@@ -1092,7 +1199,7 @@ oms_status_enu_t oms_RunFile(const char* filename_)
       status = oms_addSystem(systemName.c_str(), (kind == oms_fmi_kind_cs) ? oms_system_wc : oms_system_sc);
     if (oms_status_ok != status) return logError("oms_addSystem failed");
 
-    status = oms_addSubModel(fmuName.c_str(), filename.c_str());
+    status = oms_addSubModel(fmuName.c_str(), filename);
     if (oms_status_ok != status) return logError("oms_addSubModel failed");
 
     if (oms::Flags::ResultFile() != "<default>")
@@ -1100,7 +1207,6 @@ oms_status_enu_t oms_RunFile(const char* filename_)
     oms_setStartTime(modelName.c_str(), oms::Flags::StartTime());
     oms_setStopTime(modelName.c_str(), oms::Flags::StopTime());
     oms_setTolerance(modelName.c_str(), oms::Flags::Tolerance(), oms::Flags::Tolerance());
-    oms_setFixedStepSize(systemName.c_str(), (oms::Flags::StopTime() - oms::Flags::StartTime()) / (oms::Flags::Intervals()-1));
     if (kind == oms_fmi_kind_me_and_cs)
       oms_setSolver(systemName.c_str(), oms::Flags::DefaultModeIsCS() ? oms::Flags::MasterAlgorithm() : oms::Flags::Solver());
     else
@@ -1114,7 +1220,7 @@ oms_status_enu_t oms_RunFile(const char* filename_)
   else if (type == ".ssp")
   {
     char* cref;
-    oms_importFile(filename.c_str(), &cref);
+    oms_importFile(filename, &cref);
 
     if (oms::Flags::ResultFile() != "<default>")
       oms_setResultFile(cref, oms::Flags::ResultFile().c_str(), 1);
@@ -1133,11 +1239,17 @@ oms_status_enu_t oms_RunFile(const char* filename_)
     lua_State *L = luaL_newstate();
     luaL_openlibs(L);
     luaopen_OMSimulatorLua(L);
-    if (luaL_loadfile(L, filename.c_str()))
-      return logError(lua_tostring(L, -1));
+    if (luaL_loadfile(L, filename)) {
+      logError(lua_tostring(L, -1));
+      lua_close(L);
+      return oms_status_error;
+    }
 
-    if (lua_pcall(L, 0, 0, 0))
-      return logError(lua_tostring(L, -1));
+    if (lua_pcall(L, 0, 0, 0)) {
+      logError(lua_tostring(L, -1));
+      lua_close(L);
+      return oms_status_error;
+    }
 
     lua_close(L);
 #else
@@ -1145,7 +1257,7 @@ oms_status_enu_t oms_RunFile(const char* filename_)
 #endif
   }
   else
-    return logError("Not able to process file '" + filename + "'\nUse OMSimulator --help for more information.");
+    return logError("Not able to process file '" + std::string(filename) + "'\nUse OMSimulator --help for more information.");
 
   return oms_status_ok;
 }
@@ -1247,6 +1359,23 @@ oms_status_enu_t oms_getReal(const char* cref, double* value)
   return system->getReal(tail, *value);
 }
 
+oms_status_enu_t oms_getDirectionalDerivative(const char* unknownCref, const char* knownCref, double* value)
+{
+  oms::ComRef tail(unknownCref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  return system->getDirectionalDerivative(tail, knownCref, *value);
+}
+
 oms_status_enu_t oms_getResultFile(const char* cref_, char** filename, int* bufferSize)
 {
   oms::ComRef cref(cref_);
@@ -1465,6 +1594,35 @@ oms_status_enu_t oms_getStopTime(const char* cref, double* stopTime)
   return oms_status_ok;
 }
 
+oms_status_enu_t oms_getString(const char* cref, char** value)
+{
+  if (!value)
+    return logError("Null-pointer passed to oms_getString");
+  *value = nullptr;
+
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  std::string value_;
+  oms_status_enu_t status = system->getString(tail, value_);
+  if (oms_status_ok != status)
+    return status;
+
+  *value = oms::mallocAndCopyString(value_);
+  if (!*value)
+    return oms_status_fatal;
+  return oms_status_ok;
+}
+
 oms_status_enu_t oms_setStopTime(const char* cref, double stopTime)
 {
   oms::ComRef tail(cref);
@@ -1477,6 +1635,40 @@ oms_status_enu_t oms_setStopTime(const char* cref, double stopTime)
   return model->setStopTime(stopTime);
 }
 
+oms_status_enu_t oms_setString(const char* cref, const char* value)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  return system->setString(tail, value);
+}
+
+oms_status_enu_t oms_setUnit(const char* cref, const char* value)
+{
+  oms::ComRef tail(cref);
+  oms::ComRef front = tail.pop_front();
+
+  oms::Model* model = oms::Scope::GetInstance().getModel(front);
+  if (!model)
+    return logError_ModelNotInScope(front);
+
+  front = tail.pop_front();
+  oms::System* system = model->getSystem(front);
+  if (!system)
+    return logError_SystemNotInModel(model->getCref(), front);
+
+  return system->setUnit(tail, value);
+}
+
 oms_status_enu_t oms_getTime(const char* cref, double* time)
 {
   oms::ComRef tail(cref);
@@ -1601,34 +1793,14 @@ oms_status_enu_t oms_extractFMIKind(const char* filename, oms_fmi_kind_enu_t* ki
   if (!kind)
     return logError("Invalid argument \"kind=NULL\"");
 
-  // Usage: miniunz [-e] [-x] [-v] [-l] [-o] [-p password] file.zip [file_to_extr.] [-d extractdir]
-  //        -e  Extract without pathname (junk paths)
-  //        -x  Extract with pathname
-  //        -v  list files
-  //        -l  list files
-  //        -d  directory to extract into
-  //        -o  overwrite files without prompting
-  //        -p  extract crypted file using password
-
-  std::string cd = oms::Scope::GetInstance().getWorkingDirectory();
-  int argc = 6;
-  char **argv = new char*[argc];
-  int i=0;
-  argv[i++] = (char*)"miniunz";
-  argv[i++] = (char*)"-xo";
-  argv[i++] = (char*)filename;
-  argv[i++] = (char*)"modelDescription.xml";
-  argv[i++] = (char*)"-d";
-  argv[i++] = (char*)oms::Scope::GetInstance().getTempDirectory().c_str();
-  int status = ::miniunz(argc, argv);
-  delete[] argv;
-  oms::Scope::GetInstance().setWorkingDirectory(cd);
-
-  if (status != 0)
-    return logError("failed to extract modelDescription.xml from \"" + std::string(filename) + "\"");
+  const char* modelDescription = ::miniunz_onefile_to_memory(filename, "modelDescription.xml");
+  if (!modelDescription)
+    return logError("failed to extract \"modelDescription.xml\" from \"" + std::string(filename) + "\"");
 
   oms::Snapshot snapshot;
-  if (oms_status_ok != snapshot.importResourceFile("modelDescription.xml", oms::Scope::GetInstance().getTempDirectory()))
+  oms_status_enu_t status = snapshot.importResourceMemory("modelDescription.xml", modelDescription);
+  ::miniunz_free(modelDescription);
+  if (oms_status_ok != status)
     return logError("Failed to import");
   const pugi::xml_node node = snapshot.getResourceNode("modelDescription.xml");
 
